/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TestHelper.java,v fe6ed8e15597 2010/08/08 09:53:01 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android;

import android.app.Activity;
import android.app.Instrumentation;
import android.os.SystemClock;
import android.test.ActivityInstrumentationTestCase2;
import android.test.TouchUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewConfiguration;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ListView;
import com.jayway.android.robotium.solo.Solo;
import it.tidalwave.util.logging.Logger;
import java.util.List;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class TestHelper
  {
    private static final String CLASS = TestHelper.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    protected final int timeout = 30000;

    protected final int generalDelay = 300;

    @Nonnull
    private final ActivityInstrumentationTestCase2 instrumentationTestCase;

    @Nonnull
    private final Solo solo;

    private boolean touchMode = false;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TestHelper (final @Nonnull ActivityInstrumentationTestCase2 instrumentationTestCase,
                       final @Nonnull Solo solo)
      {
        this.instrumentationTestCase = instrumentationTestCase;
        this.solo = solo;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void waitForListNotEmpty (final int id)
      throws InterruptedException
      {
        final long time = System.currentTimeMillis();
        final Activity activity = solo.getCurrentActivity();
        final ListView listView = (ListView)activity.findViewById(id);

        if (listView == null)
          {
            throw new AssertionError("No ListView with id " + id);
          }

        while ((listView.getAdapter() == null) || listView.getAdapter().isEmpty())
          {
            if (System.currentTimeMillis() - time >= timeout)
              {
                throw new AssertionError("Time out while waiting for a ListView to become not empty");
              }

            logger.fine("Waiting for ListView #%d to become not empty...", id);
            Thread.sleep(100);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void waitForVisible (final int id)
      throws InterruptedException
      {
        final long time = System.currentTimeMillis();
        final Activity activity = solo.getCurrentActivity();
        final View view = activity.findViewById(id);

        while (view.getVisibility() != View.VISIBLE)
          {
            if (System.currentTimeMillis() - time >= timeout)
              {
                throw new AssertionError("Time out while waiting for a View to become visible");
              }

            logger.info("Waiting for View #%d to become visible...", id);
            Thread.sleep(100);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void waitForActivity (final @Nonnull Class<? extends Activity> activityClass)
      throws InterruptedException
      {
        final long time = System.currentTimeMillis();

        while (!activityClass.isAssignableFrom(solo.getCurrentActivity().getClass()))
          {
            if (System.currentTimeMillis() - time >= timeout)
              {
                throw new AssertionError("Time out while waiting for " + activityClass.getName());
              }
            
            logger.info("Waiting for %s ... (%s)", activityClass.getName(), solo.getCurrentActivity().getClass().getName());
            Thread.sleep(100);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void clickOnImageButtonById (final int id)
      {
        if (!touchMode)
          {
            final ImageButton button = (ImageButton)solo.getCurrentActivity().findViewById(id);

            if (button == null)
              {
                throw new AssertionError("Cannot find button #" + id + " on " + solo.getCurrentActivity());
              }

            solo.sleep(generalDelay);
            requestFocus(button);
            instrumentationTestCase.sendKeys(KeyEvent.KEYCODE_DPAD_CENTER);
          }
        else
          {
            final List<ImageButton> buttons = solo.getCurrentImageButtons();

            for (int i = 0; i < buttons.size(); i++)
              {
                final ImageButton button = buttons.get(i);

                if (button.getId() == id)
                  {
                    logger.info(">>>> Clicking on ImageButton #%d  ...", i);
                    clickViewWithTouchUtils(button);                      
//                    solo.clickOnImageButton(i);
                    return;
                  }
              }

            throw new AssertionError("Cannot find button #" + id + " on " + solo.getCurrentActivity() + ": " + buttons);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void clickOnButtonById (final int id)
      {
        final List<Button> buttons = solo.getCurrentButtons();

        for (int i = 0; i < buttons.size(); i++)
          {
            final Button button = buttons.get(i);

            if (button.getId() == id)
              {
                logger.info(">>>> Clicking on Button #%d ...", i);

                if (touchMode)
                  {
                    clickViewWithTouchUtils(button);
//                    solo.clickOnButton(i);
                  }
                else
                  {
                    solo.sleep(generalDelay);
                    requestFocus(button);
                    press(KeyEvent.KEYCODE_DPAD_CENTER);
                  }
                
                return;
              }
          }

        throw new AssertionError("Cannot find button #" + id + " on " + solo.getCurrentActivity() + ": " + buttons);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void clickOnListView (final @Nonnull ListView listView, final @Nonnegative int index)
      {
        if (touchMode)
          {
            clickViewWithTouchUtils(listView.getChildAt(index));
//            solo.clickInList(listIndex, index);
          }
        else
          {
            if (true) // scroll down
              {
                requestFocus(listView);
                final Instrumentation instrumentation = instrumentationTestCase.getInstrumentation();

                for (int i = 0; i < index; i++)
                  {
                    instrumentationTestCase.sendKeys(KeyEvent.KEYCODE_DPAD_DOWN);
                    instrumentation.waitForIdleSync();
                    solo.sleep(generalDelay);
                  }

                press(KeyEvent.KEYCODE_DPAD_CENTER);
              }

            else // immediate click - doesn't work, doesn't put the focus in the proper place
              {
                requestFocus(listView);
                requestFocus(listView.getChildAt(index));
                press(KeyEvent.KEYCODE_DPAD_CENTER);
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void requestFocus (final @Nonnull View view)
      {
        logger.info("Requesting focus for %s...", view);

        if (!view.isFocused())
          {
            instrumentationTestCase.getActivity().runOnUiThread(new Runnable()
              {
                public void run()
                  {
                    view.requestFocus();
                  }
              });

            instrumentationTestCase.getInstrumentation().waitForIdleSync();
            ActivityInstrumentationTestCase2.assertTrue(view.isFocused());
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void press (final int keyCode)
      {
        long downTime = SystemClock.uptimeMillis();
        long eventTime = SystemClock.uptimeMillis();
        final KeyEvent keyDown = new KeyEvent(downTime, eventTime, KeyEvent.ACTION_DOWN, keyCode, 0);
        final Instrumentation instrumentation = instrumentationTestCase.getInstrumentation();
        instrumentation.sendKeySync(keyDown);
        solo.sleep((int)(ViewConfiguration.getLongPressTimeout() * 0.8f));
        downTime = SystemClock.uptimeMillis();
        eventTime = SystemClock.uptimeMillis();
        final KeyEvent keyUp = new KeyEvent(downTime, eventTime, KeyEvent.ACTION_UP, keyCode, 0);
        instrumentation.sendKeySync(keyUp);
        instrumentation.waitForIdleSync();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void sendFilterKeyToListView (final @Nonnull ListView listView, final int c)
      {
        logger.info("sending filter key %d to %s...", c, listView);
        requestFocus(listView);
        solo.sleep(500);
        final Instrumentation instrumentation = instrumentationTestCase.getInstrumentation();
        instrumentation.sendCharacterSync(c);
        instrumentation.waitForIdleSync();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void sendFilterKeyToListView (final @Nonnull ListView listView, final @Nonnull String chars)
      {
        logger.info("sending filter keys %s to %s...", chars, listView);
        requestFocus(listView);
        final Instrumentation instrumentation = instrumentationTestCase.getInstrumentation();

        for (int i = 0; i < chars.length(); i++)
          {
            instrumentation.sendCharacterSync(chars.charAt(i) - 'a' + KeyEvent.KEYCODE_A);
            instrumentation.waitForIdleSync();
            solo.sleep(500);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void clickViewWithTouchUtils (final @Nonnull View view)
      {
//        solo.getCurrentActivity().runOnUiThread(new Runnable()
//          {
//            public void run()
//              {
                TouchUtils.tapView(instrumentationTestCase, view);
//              }
//          });
      }
  }
